from django.contrib.auth.models import AbstractBaseUser, PermissionsMixin
from django.contrib.auth.models import BaseUserManager
from django.db import models
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
from django.utils.six import python_2_unicode_compatible, text_type


class EmailUserQuerySet(models.query.QuerySet):
    def _filter_or_exclude(self, *args, **kwargs):
        """ Make email lookups case-insensitive """
        if 'email' in kwargs:
            kwargs['email__iexact'] = kwargs.pop('email')
        return super(EmailUserQuerySet, self)._filter_or_exclude(*args, **kwargs)

    def active(self):
        return self.filter(is_active=True)


class EmailUserManager(BaseUserManager):
    def get_queryset(self):
        return EmailUserQuerySet(self.model, using=self._db)

    def create_user(self, email, password=None, **extra_fields):
        if not email:
            raise ValueError('Users must have an email address')
        user = self.model(email=self.normalize_email(email), **extra_fields)
        user.set_password(password)
        user.validate_unique()
        user.save(using=self._db)
        return user

    def create_superuser(self, email, password, **extra_fields):
        user = self.create_user(email, password=password, **extra_fields)
        user.is_active = True
        user.is_staff = True
        user.is_superuser = True
        user.validate_unique()
        user.save(using=self._db)
        return user

    def active(self):
        return self.get_queryset().active()


@python_2_unicode_compatible
class EmailUser(AbstractBaseUser, PermissionsMixin):
    email = models.EmailField(_('email address'), unique=True, max_length=255, db_index=True)
    first_name = models.CharField(_('first name'), max_length=255)
    last_name = models.CharField(_('last name'), max_length=255)
    date_joined = models.DateTimeField(_('date joined'), default=timezone.now)
    is_active = models.BooleanField(_('active'), default=True)
    is_staff = models.BooleanField(_('staff status'), default=False)
    created = models.DateTimeField(_('created'), default=timezone.now, editable=False)

    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = ()

    objects = EmailUserManager()

    class Meta:
        abstract = True
        ordering = ('email',)

    def __str__(self):
        return self.get_full_name() or self.email

    def get_full_name(self):
        if not self.first_name and not self.last_name:
            return ''
        return text_type(' ').join([self.first_name, self.last_name])

    def get_short_name(self):
        return self.first_name



from django import forms
from django.contrib.auth import get_user_model
from django.contrib.auth.forms import ReadOnlyPasswordHashField
from django.utils.translation import ugettext_lazy as _


class EmailUserCreationForm(forms.ModelForm):
    password1 = forms.CharField(label=_('Password'), widget=forms.PasswordInput)
    password2 = forms.CharField(
        label=_('Password confirmation'), widget=forms.PasswordInput)

    class Meta:
        model = get_user_model()
        fields = ('email', 'first_name', 'last_name')

    def clean_password2(self):
        password1 = self.cleaned_data.get('password1')
        password2 = self.cleaned_data.get('password2')
        if password1 and password2 and password1 != password2:
            raise forms.ValidationError(_('Passwords don\'t match'))
        return password2

    def save(self, commit=True):
        user = super(UserCreationForm, self).save(commit=False)
        user.set_password(self.cleaned_data['password1'])
        if commit:
            user.save()
        return user


class EmailUserChangeForm(forms.ModelForm):
    password = ReadOnlyPasswordHashField(
        label=_('Password'),
        help_text=(
            'Raw passwords are not stored, so there is no way to see '
            'this user\'s password, but you can change the password '
            'using <a href="../password/">this form</a>.'
        ),
    )

    class Meta:
        model = get_user_model()
        fields = ('password',)

    def clean_password(self):
        return self.initial['password']



from django.contrib.auth.admin import UserAdmin as DjangoUserAdmin
from django.utils.translation import ugettext_lazy as _
from . import forms


class EmailUserAdmin(DjangoUserAdmin):
    form = forms.EmailUserChangeForm
    add_form = forms.EmailUserCreationForm

    list_display = ('email', 'first_name', 'last_name', 'is_staff')
    fieldsets = (
        (None, {'fields': ('email', 'password',)}),
        (_('Personal info'), {'fields': ('first_name', 'last_name',)}),
        (_('Permissions'), {'fields': (
            'is_active', 'is_staff', 'is_superuser',
            'groups', 'user_permissions')}),
        (_('Important dates'), {'fields': ('last_login', 'date_joined')}),
    )
    add_fieldsets = (
        (
            None, {
                'classes': ('wide',),
                'fields': (
                    'email', 'first_name', 'last_name',
                    'password1', 'password2',
                )
            }
        ),
    )
    search_fields = ('email', 'first_name', 'last_name')
    ordering = ('email',)




import setuptools
import email_user

setuptools.setup(
    name='django-email-user',
    version=email_user.__version__,
    author='Interaction Consortium',
    author_email='studio@interaction.net.au',
    url='https://github.com/ixc/django-email-user',
    description='Abstract models + utils to replace `django.contrib.auth.User` with a model that uses an email '
                'address instead of a username',
    license='MIT',
    packages=setuptools.find_packages(),
    include_package_data=True,
)